@@ -40,7 +40,7 @@ from __future__ import with_statement
 from contextlib import contextmanager
 
 from mapproxy.grid import MetaGrid
-from mapproxy.image import merge_images
+from mapproxy.image.merge import merge_images
 from mapproxy.image.tile import TileSplitter
 from mapproxy.layer import MapQuery, BlankImage
 from mapproxy.util import async
@@ -255,8 +255,10 @@ class TileCreator(object):
                 # call as_buffer to force conversion into cache format
                 source.as_buffer(self.tile_mgr.image_opts)
                 tile.source = source
+                tile.cacheable = source.cacheable
                 tile = self.tile_mgr.apply_tile_filter(tile)
-                self.cache.store_tile(tile)
+                if source.cacheable:
+                    self.cache.store_tile(tile)
             else:
                 self.cache.load_tile(tile)
         return [tile]
@@ -308,7 +310,8 @@ class TileCreator(object):
                 splitted_tiles = split_meta_tiles(meta_tile_image, meta_tile.tile_patterns,
                                                   tile_size, self.tile_mgr.image_opts)
                 splitted_tiles = map(self.tile_mgr.apply_tile_filter, splitted_tiles)
-                self.cache.store_tiles(splitted_tiles)
+                if meta_tile_image.cacheable:
+                    self.cache.store_tiles(splitted_tiles)
                 return splitted_tiles
         # else
         tiles = [Tile(coord) for coord in meta_tile.tiles]
@@ -322,14 +325,29 @@ class Tile(object):
     :ivar source: the data of this tile
     :type source: ImageSource
     """
-    def __init__(self, coord, source=None):
+    def __init__(self, coord, source=None, cacheable=True):
         self.coord = coord
         self.source = source
         self.location = None
         self.stored = False
+        self._cacheable = cacheable
         self.size = None
         self.timestamp = None
     
+    def _cacheable_get(self):
+        return CacheInfo(cacheable=self._cacheable, timestamp=self.timestamp,
+            size=self.size)
+
+    def _cacheable_set(self, cacheable):
+        if isinstance(cacheable, bool):
+            self._cacheable = cacheable
+        else: # assume cacheable is CacheInfo
+            self._cacheable = cacheable.cacheable
+            self.timestamp = cacheable.timestamp
+            self.size = cacheable.size
+
+    cacheable = property(_cacheable_get, _cacheable_set)
+
     def source_buffer(self, *args, **kw):
         if self.source is not None:
             return self.source.as_buffer(*args, **kw)
@@ -390,6 +408,15 @@ class Tile(object):
     def __repr__(self):
         return 'Tile(%r, source=%r)' % (self.coord, self.source)
 
+class CacheInfo(object):
+    def __init__(self, cacheable=True, timestamp=None, size=None):
+        self.cacheable = cacheable
+        self.timestamp = timestamp
+        self.size = size
+
+    def __nonzero__(self):
+        return self.cacheable
+
 class TileCollection(object):
     def __init__(self, tile_coords):
         self.tiles = [Tile(coord) for coord in tile_coords]
@@ -442,7 +469,7 @@ def split_meta_tiles(meta_tile, tiles, tile_size, image_opts):
         tile_coord, crop_coord = tile
         if tile_coord is None: continue
         data = splitter.get_tile(crop_coord, tile_size)
-        new_tile = Tile(tile_coord)
+        new_tile = Tile(tile_coord, cacheable=meta_tile.cacheable)
         new_tile.source = data
         split_tiles.append(new_tile)
     return split_tiles
