@@ -14,7 +14,7 @@
 # limitations under the License.
 
 from mapproxy.util.ext.dictspec.validator import validate, ValidationError
-from mapproxy.util.ext.dictspec.spec import one_off, anything, number
+from mapproxy.util.ext.dictspec.spec import one_of, anything, number
 from mapproxy.util.ext.dictspec.spec import recursive, required, type_spec, combined
 
 
@@ -33,7 +33,7 @@ def validate_mapproxy_conf(conf_dict):
 coverage = {
     'polygons': str(),
     'polygons_srs': str(),
-    'bbox': one_off(str(), [number()]),
+    'bbox': one_of(str(), [number()]),
     'bbox_srs': str(),
     'ogr_datasource': str(),
     'ogr_where': str(),
@@ -98,6 +98,61 @@ cache_types = {
     },
 }
 
+on_error = {
+    anything(): {
+        required('response'): one_of([int], str),
+        'cache': bool,
+    }
+}
+
+wms_130_layer_md = {
+    'abstract': basestring,
+    'keyword_list': [
+        {
+            'vocabulary': basestring,
+            'keywords': [basestring],
+        }
+    ],
+    'attribution': {
+        'title': basestring,
+        'url':    str,
+        'logo': {
+            'url':    str,
+            'width':  int,
+            'height': int,
+            'format': basestring,
+       }
+    },
+    'identifier': [
+        {
+            'url': str,
+            'name': basestring,
+            'value': basestring,
+        }
+    ],
+    'metadata': [
+        {
+            'url': str,
+            'type': str,
+            'format': str,
+        },
+    ],
+    'data': [
+        {
+            'url': str,
+            'format': str,
+        }
+
+    ],
+    'feature_list': [
+        {
+            'url': str,
+            'format': str,
+        }
+    ],
+}
+
+
 mapproxy_yaml_spec = {
     'globals': {
         'image': {
@@ -139,11 +194,11 @@ mapproxy_yaml_spec = {
             'base': str(),
             'name': str(),
             'srs': str(),
-            'bbox': one_off(str(), [number()]),
+            'bbox': one_of(str(), [number()]),
             'bbox_srs': str(),
             'num_levels': int(),
             'res': [number()],
-            'res_factor': one_off(number(), str()),
+            'res_factor': one_of(number(), str()),
             'max_res': number(),
             'min_res': number(),
             'stretch_factor': number(),
@@ -174,7 +229,7 @@ mapproxy_yaml_spec = {
             'watermark': {
                 'text': basestring,
                 'font_size': number(),
-                'color': one_off(str(), [number()]),
+                'color': one_of(str(), [number()]),
                 'opacity': number(),
                 'spacing': str(),
             },
@@ -202,7 +257,7 @@ mapproxy_yaml_spec = {
                 anything(): str()
             },
             'on_source_errors': str(),
-            'max_output_pixels': one_off(number(), [number()]),
+            'max_output_pixels': one_of(number(), [number()]),
             'strict': bool(),
             'md': {
                 'title': basestring,
@@ -229,7 +284,7 @@ mapproxy_yaml_spec = {
                 },
                 'image': combined(image_opts, {
                     'opacity':number(),
-                    'transparent_color': one_off(str(), [number()]),
+                    'transparent_color': one_of(str(), [number()]),
                     'transparent_color_tolerance': number(),
                 }),
                 'supported_formats': [str()],
@@ -247,13 +302,18 @@ mapproxy_yaml_spec = {
                         'featureinfo': bool(),
                         'legendgraphic': bool(),
                         'legendurl': str(),
+                        'featureinfo_format': str(),
+                        'featureinfo_xslt': str(),
                     },
                     'image': combined(image_opts, {
                         'opacity':number(),
-                        'transparent_color': one_off(str(), [number()]),
+                        'transparent_color': one_of(str(), [number()]),
                         'transparent_color_tolerance': number(),
                     }),
-                    'req': {
+                    'supported_formats': [str()],
+                    'supported_srs': [str()],
+                    required('req'): {
+                        required('map'): str(),
                         anything(): anything()
                     },
                     'mapserver': mapserver_opts,
@@ -264,14 +324,15 @@ mapproxy_yaml_spec = {
                 'image': image_opts,
                 'grid': str(),
                 'request_format': str(),
-                'origin': str(),
+                'origin': str(), # TODO: remove with 1.5
                 'http': http_opts,
+                'on_error': on_error,
             }),
             'mapnik': combined(source_commons, {
                 required('mapfile'): str(),
                 'transparent': bool(),
                 'image': image_opts,
-                'layers': one_off(str(), [str()]),
+                'layers': one_of(str(), [str()]),
                 'use_mapnik2': bool(),
             }),
             'debug': {
@@ -279,12 +340,13 @@ mapproxy_yaml_spec = {
         })
     },
     
-    'layers': one_off(
+    'layers': one_of(
         {
             anything(): combined(scale_hints, {
                 'sources': [str()],
                 required('title'): basestring,
                 'legendurl': str(),
+                'md': wms_130_layer_md,
             })
         },
         recursive([combined(scale_hints, {
@@ -292,8 +354,8 @@ mapproxy_yaml_spec = {
             'name': str(),
             required('title'): basestring,
             'legendurl': str(),
-            'layers': [recursive()],
-            
+            'layers': recursive(),
+            'md': wms_130_layer_md,
         })])
     ),
 }
