@@ -958,10 +958,10 @@ def bbox_contains(one, two):
     b_x0, b_y0, b_x1, b_y1 = two
     
     if (
-        a_x0 < b_x0 and
-        a_x1 > b_x1 and
-        a_y0 < b_y0 and
-        a_y1 > b_y1
+        a_x0 <= b_x0 and
+        a_x1 >= b_x1 and
+        a_y0 <= b_y0 and
+        a_y1 >= b_y1
         ): return True
     
     return False
