@@ -183,6 +183,7 @@ class WMSMapRequest(WMSRequest):
                       'width', 'height', 'format']
     #pylint: disable-msg=E1102
     xml_exception_handler = None
+    prevent_image_exception = False
 
     def __init__(self, param=None, url='', validate=False, non_strict=False, **kw):
         WMSRequest.__init__(self, param=param, url=url, validate=validate,
@@ -214,7 +215,7 @@ class WMSMapRequest(WMSRequest):
                                request=self)
     
     def validate_format(self, image_formats):
-        format = self.params['format'].split(';', 1)[0].strip()
+        format = self.params['format']
         if format not in image_formats:
             format = self.params['format']
             self.params['format'] = 'image/png'
@@ -234,6 +235,8 @@ class WMSMapRequest(WMSRequest):
     
     @property
     def exception_handler(self):
+        if self.prevent_image_exception:
+            return self.xml_exception_handler()
         if 'exceptions' in self.params:
             if 'image' in self.params['exceptions'].lower():
                 return exception.WMSImageExceptionHandler()
@@ -297,15 +300,17 @@ class WMS111MapRequest(WMSMapRequest):
     def adapt_to_111(self):
         del self.params['wmtver']
 
-def _switch_bbox(self):
-    if self.bbox is not None and self.srs is not None:
+def switch_bbox_epsg_axis_order(bbox, srs):
+    if bbox is not None and srs is not None:
         try:
-            if SRS(self.srs).is_axis_order_ne:
-                bbox = self.bbox
-                bbox = bbox[1], bbox[0], bbox[3], bbox[2]
-                self.bbox = bbox
+            if SRS(srs).is_axis_order_ne:
+                return bbox[1], bbox[0], bbox[3], bbox[2]
         except RuntimeError:
-            log.warn('unknown SRS %s' % self.srs)
+            log.warn('unknown SRS %s' % srs)
+    return bbox
+    
+def _switch_bbox(self):
+    self.bbox = switch_bbox_epsg_axis_order(self.bbox, self.srs)
 
 class WMS130MapRequestParams(WMSMapRequestParams):
     """
