@@ -1,18 +1,17 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2010 Omniscale <http://omniscale.de>
 # 
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 # 
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Affero General Public License for more details.
+#    http://www.apache.org/licenses/LICENSE-2.0
 # 
-# You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 """
 Service exception handling (WMS exceptions, XML, in_image, etc.).
@@ -20,6 +19,7 @@ Service exception handling (WMS exceptions, XML, in_image, etc.).
 from mapproxy.exception import ExceptionHandler, XMLExceptionHandler
 from mapproxy.response import Response
 from mapproxy.image.message import message_image
+from mapproxy.image.opts import ImageOptions
 import mapproxy.service
 from mapproxy.template import template_loader
 get_template = template_loader(mapproxy.service.__file__, 'templates')
@@ -69,8 +69,8 @@ class WMSImageExceptionHandler(ExceptionHandler):
         transparent = ('transparent' in params
                        and params['transparent'].lower() == 'true')
         bgcolor = WMSImageExceptionHandler._bgcolor(request.params)
-        result = message_image(request_error.message, size=size, format=format,
-                               bgcolor=bgcolor, transparent=transparent)
+        image_opts = ImageOptions(format=format, bgcolor=bgcolor, transparent=transparent)
+        result = message_image(request_error.msg, size=size, image_opts=image_opts)
         return Response(result.as_buffer(), content_type=params.format_mime_type)
     
     @staticmethod
@@ -95,5 +95,5 @@ class WMSBlankExceptionHandler(WMSImageExceptionHandler):
     """
     
     def render(self, request_error):
-        request_error.message = ''
+        request_error.msg = ''
         return WMSImageExceptionHandler.render(self, request_error)
