@@ -28,15 +28,16 @@ from mapproxy.util import reraise_exception
 import logging
 log = logging.getLogger('mapproxy.source.tile')
 log_config = logging.getLogger('mapproxy.config')
+
 class TiledSource(Source):
-    def __init__(self, grid, client, inverse=False, coverage=None, image_opts=None):
+    def __init__(self, grid, client, coverage=None, image_opts=None, error_handler=None):
         Source.__init__(self, image_opts=image_opts)
         self.grid = grid
         self.client = client
-        self.inverse = inverse
         self.image_opts = image_opts or ImageOptions()
         self.coverage = coverage
         self.extent = coverage.extent if coverage else map_extent_from_grid(grid)
+        self.error_handler = error_handler
     
     def get_map(self, query):
         if self.grid.tile_size != query.size:
@@ -65,10 +66,12 @@ class TiledSource(Source):
 
         tile_coord = tiles.next()
         
-        if self.inverse:
-            tile_coord = self.grid.flip_tile_coord(tile_coord)
         try:
             return self.client.get_tile(tile_coord, format=query.format)
         except HTTPClientError, e:
+            if self.error_handler:
+                resp = self.error_handler.handle(e.response_code, query)
+                if resp:
+                    return resp
             log.warn('could not retrieve tile: %s', e)
             reraise_exception(SourceError(e.args[0]), sys.exc_info())
\ No newline at end of file
